#include <stdio.h>

#include "http.h"
#include "http_loop.h"

http_loop_gctx *http_loop_gctx_init() {
  http_loop_gctx *gctx = calloc(sizeof(http_loop_gctx), 1);

  return gctx;
}

http_loop_ctx *http_loop_ctx_init(http_loop_gctx *g) {
  http_loop_ctx *ctx = calloc(sizeof(http_loop_ctx), 1);
  ctx->g = g;

  return ctx;
}

void http_loop_ctx_free(http_loop_ctx *ctx) {
  http_loop_ctx_reset(ctx);

  free(ctx);
}

void http_loop_ctx_reset(http_loop_ctx *ctx) {
  ctx->route = NULL;
  ctx->current_step = 0;

  if (ctx->req.body_type == http_body_buf && ctx->req.body.buf != NULL) {
    free(ctx->req.body.buf);
    ctx->req.body.buf = NULL;
  } else if (ctx->req.body_type == http_body_file &&
             ctx->req.body.file != NULL) {
    fclose(ctx->req.body.file);
    ctx->req.body.file = NULL;
  }

  if (ctx->res.head != NULL) {
    free((void *)ctx->res.head);
    ctx->res.head = NULL;
  }

  if (ctx->res.body_type == http_body_buf && ctx->res.body.buf != NULL) {
    free(ctx->res.body.buf);
    ctx->res.body.buf = NULL;
  } else if (ctx->res.body_type == http_body_file &&
             ctx->res.body.file != NULL) {
    fclose(ctx->res.body.file);
    ctx->res.body.file = NULL;
  }

  for (size_t i = 0; i < ctx->res.header_count; i++) {
    if (ctx->res.headers[i].owned) {
      free((void *)ctx->res.headers[i].value);
    }
  }

  ctx->res.header_count = 0;

  ctx->res.status = 0;
  ctx->res.head_len = 0;
  ctx->res.head_written = 0;
  ctx->res.body_len = 0;
  ctx->res.body_written = 0;
  ctx->res.owns_body = false;
}
